import { highlightSpecialChars, drawSelection, highlightActiveLine, keymap, ViewPlugin, PluginField, EditorView } from '@codemirror/view';
import { EditorState, Facet, StateEffect, StateField, Compartment } from '@codemirror/state';
import { history, historyKeymap } from '@codemirror/history';
import { foldGutter, foldKeymap } from '@codemirror/fold';
import { indentOnInput, syntaxTree, LanguageSupport } from '@codemirror/language';
import { lineNumbers } from '@codemirror/gutter';
import { defaultKeymap } from '@codemirror/commands';
import { bracketMatching } from '@codemirror/matchbrackets';
import { closeBrackets, closeBracketsKeymap } from '@codemirror/closebrackets';
import { highlightSelectionMatches, searchKeymap } from '@codemirror/search';
import { autocompletion, completionKeymap, startCompletion } from '@codemirror/autocomplete';
import { commentKeymap } from '@codemirror/comment';
import { rectangularSelection } from '@codemirror/rectangular-selection';
import { defaultHighlightStyle, HighlightStyle, tags } from '@codemirror/highlight';
import { lintKeymap, setDiagnostics } from '@codemirror/lint';
import React, { useEffect, useState, useRef, forwardRef, useImperativeHandle } from 'react';
import { dequal } from 'dequal';
import { javascript } from '@codemirror/lang-javascript';
import { json } from '@codemirror/lang-json';
import { StandardSQL, schemaCompletion, keywordCompletion } from '@codemirror/lang-sql';

/// This is an extension value that just pulls together a whole lot of
/// extensions that you might want in a basic editor. It is meant as a
/// convenient helper to quickly set up CodeMirror without installing
/// and importing a lot of packages.
///
/// Specifically, it includes...
///
///  - [the default command bindings](#commands.defaultKeymap)
///  - [line numbers](#gutter.lineNumbers)
///  - [special character highlighting](#view.highlightSpecialChars)
///  - [the undo history](#history.history)
///  - [a fold gutter](#fold.foldGutter)
///  - [custom selection drawing](#view.drawSelection)
///  - [multiple selections](#state.EditorState^allowMultipleSelections)
///  - [reindentation on input](#language.indentOnInput)
///  - [the default highlight style](#highlight.defaultHighlightStyle) (as fallback)
///  - [bracket matching](#matchbrackets.bracketMatching)
///  - [bracket closing](#closebrackets.closeBrackets)
///  - [autocompletion](#autocomplete.autocompletion)
///  - [rectangular selection](#rectangular-selection.rectangularSelection)
///  - [active line highlighting](#view.highlightActiveLine)
///  - [selection match highlighting](#search.highlightSelectionMatches)
///  - [search](#search.searchKeymap)
///  - [commenting](#comment.commentKeymap)
///  - [linting](#lint.lintKeymap)
///
/// (You'll probably want to add some language package to your setup
/// too.)
///
/// This package does not allow customization. The idea is that, once
/// you decide you want to configure your editor more precisely, you
/// take this package's source (which is just a bunch of imports and
/// an array literal), copy it into your own code, and adjust it as
/// desired.
const basicSetup = [
    lineNumbers(),
    highlightSpecialChars(),
    history(),
    foldGutter(),
    drawSelection(),
    EditorState.allowMultipleSelections.of(true),
    indentOnInput(),
    defaultHighlightStyle.fallback,
    bracketMatching(),
    closeBrackets(),
    autocompletion({
        activateOnTyping: true,
        // override
    }),
    rectangularSelection(),
    highlightActiveLine(),
    highlightSelectionMatches(),
    keymap.of([
        ...closeBracketsKeymap,
        ...defaultKeymap,
        ...searchKeymap,
        ...historyKeymap,
        ...foldKeymap,
        ...commentKeymap,
        ...completionKeymap,
        ...lintKeymap,
    ]),
];

/**
 * hooks for mount callback
 * @param cb
 * @returns
 */
function useMount(cb) {
    useEffect(() => {
        cb && cb();
    }, []);
    return null;
}
/**
 * hook for unmount callback
 * @param cb
 * @returns
 */
function useUnmount(cb) {
    useEffect(() => {
        return () => {
            cb();
        };
    }, []);
    return null;
}

const panelConfig = /*@__PURE__*/Facet.define({
    combine(configs) {
        let topContainer, bottomContainer;
        for (let c of configs) {
            topContainer = topContainer || c.topContainer;
            bottomContainer = bottomContainer || c.bottomContainer;
        }
        return { topContainer, bottomContainer };
    }
});
const panelPlugin = /*@__PURE__*/ViewPlugin.fromClass(class {
    constructor(view) {
        this.input = view.state.facet(showPanel);
        this.specs = this.input.filter(s => s);
        this.panels = this.specs.map(spec => spec(view));
        let conf = view.state.facet(panelConfig);
        this.top = new PanelGroup(view, true, conf.topContainer);
        this.bottom = new PanelGroup(view, false, conf.bottomContainer);
        this.top.sync(this.panels.filter(p => p.top));
        this.bottom.sync(this.panels.filter(p => !p.top));
        for (let p of this.panels) {
            p.dom.classList.add("cm-panel");
            // FIXME drop on next breaking release
            if (p.class)
                p.dom.classList.add(p.class);
            if (p.mount)
                p.mount();
        }
    }
    update(update) {
        let conf = update.state.facet(panelConfig);
        if (this.top.container != conf.topContainer) {
            this.top.sync([]);
            this.top = new PanelGroup(update.view, true, conf.topContainer);
        }
        if (this.bottom.container != conf.bottomContainer) {
            this.bottom.sync([]);
            this.bottom = new PanelGroup(update.view, false, conf.bottomContainer);
        }
        this.top.syncClasses();
        this.bottom.syncClasses();
        let input = update.state.facet(showPanel);
        if (input != this.input) {
            let specs = input.filter(x => x);
            let panels = [], top = [], bottom = [], mount = [];
            for (let spec of specs) {
                let known = this.specs.indexOf(spec), panel;
                if (known < 0) {
                    panel = spec(update.view);
                    mount.push(panel);
                }
                else {
                    panel = this.panels[known];
                    if (panel.update)
                        panel.update(update);
                }
                panels.push(panel);
                (panel.top ? top : bottom).push(panel);
            }
            this.specs = specs;
            this.panels = panels;
            this.top.sync(top);
            this.bottom.sync(bottom);
            for (let p of mount) {
                p.dom.classList.add("cm-panel");
                // FIXME drop on next breaking release
                if (p.class)
                    p.dom.classList.add(p.class);
                if (p.mount)
                    p.mount();
            }
        }
        else {
            for (let p of this.panels)
                if (p.update)
                    p.update(update);
        }
    }
    destroy() {
        this.top.sync([]);
        this.bottom.sync([]);
    }
}, {
    provide: /*@__PURE__*/PluginField.scrollMargins.from(value => ({ top: value.top.scrollMargin(), bottom: value.bottom.scrollMargin() }))
});
class PanelGroup {
    constructor(view, top, container) {
        this.view = view;
        this.top = top;
        this.container = container;
        this.dom = undefined;
        this.classes = "";
        this.panels = [];
        this.syncClasses();
    }
    sync(panels) {
        this.panels = panels;
        this.syncDOM();
    }
    syncDOM() {
        if (this.panels.length == 0) {
            if (this.dom) {
                this.dom.remove();
                this.dom = undefined;
            }
            return;
        }
        if (!this.dom) {
            this.dom = document.createElement("div");
            this.dom.className = this.top ? "cm-panels cm-panels-top" : "cm-panels cm-panels-bottom";
            this.dom.style[this.top ? "top" : "bottom"] = "0";
            let parent = this.container || this.view.dom;
            parent.insertBefore(this.dom, this.top ? parent.firstChild : null);
        }
        let curDOM = this.dom.firstChild;
        for (let panel of this.panels) {
            if (panel.dom.parentNode == this.dom) {
                while (curDOM != panel.dom)
                    curDOM = rm(curDOM);
                curDOM = curDOM.nextSibling;
            }
            else {
                this.dom.insertBefore(panel.dom, curDOM);
            }
        }
        while (curDOM)
            curDOM = rm(curDOM);
    }
    scrollMargin() {
        return !this.dom || this.container ? 0
            : Math.max(0, this.top ?
                this.dom.getBoundingClientRect().bottom - Math.max(0, this.view.scrollDOM.getBoundingClientRect().top) :
                Math.min(innerHeight, this.view.scrollDOM.getBoundingClientRect().bottom) - this.dom.getBoundingClientRect().top);
    }
    syncClasses() {
        if (!this.container || this.classes == this.view.themeClasses)
            return;
        for (let cls of this.classes.split(" "))
            if (cls)
                this.container.classList.remove(cls);
        for (let cls of (this.classes = this.view.themeClasses).split(" "))
            if (cls)
                this.container.classList.add(cls);
    }
}
function rm(node) {
    let next = node.nextSibling;
    node.remove();
    return next;
}
const baseTheme$1 = /*@__PURE__*/EditorView.baseTheme({
    ".cm-panels": {
        boxSizing: "border-box",
        position: "sticky",
        left: 0,
        right: 0
    },
    "&light .cm-panels": {
        backgroundColor: "#f5f5f5",
        color: "black"
    },
    "&light .cm-panels-top": {
        borderBottom: "1px solid #ddd"
    },
    "&light .cm-panels-bottom": {
        borderTop: "1px solid #ddd"
    },
    "&dark .cm-panels": {
        backgroundColor: "#333338",
        color: "white"
    }
});
/**
Opening a panel is done by providing a constructor function for
the panel through this facet. (The panel is closed again when its
constructor is no longer provided.) Values of `null` are ignored.
*/
const showPanel = /*@__PURE__*/Facet.define({
    enables: [panelPlugin, baseTheme$1]
});

class Viewbox {
    constructor(parent) {
        this.node = document.createElement("div");
    }
    appendStyle(cssStyle) {
        for (const prop in cssStyle) {
            this.node.style[prop] = cssStyle[prop];
        }
    }
    scroll(scrollTop) {
        this.node.style.transform = `translateY(${Math.round(scrollTop)}px)`;
    }
    hide() {
        this.node.style.visibility = "0";
    }
    show() {
        this.node.style.visibility = "1";
    }
    resize(width, height) {
        this.node.style.width = String(width) + "px";
        this.node.style.height = String(height) + "px";
    }
    attach(parent) {
        parent.appendChild(this.node);
    }
}
class CanvasElement {
    constructor(parent) {
        this.initLayer();
        this.attachCanvasLayer(parent);
    }
    attachCanvasLayer(parent) {
        parent.appendChild(this.node);
    }
    initLayer() {
        this.node = document.createElement("canvas");
        this.ctx = this.node.getContext("2d");
        this.node.className = "cm-minimap-layer";
        this.node.style.zIndex = "0";
    }
    resize(width, height) {
        this.node.style.width = String(width) + "px";
        this.node.style.height = String(height) + "px";
    }
}
class MinimapElement {
    constructor(view) {
        this.view = view;
        this.node = document.createElement("div");
        this.node.className = "cm-minimap";
        this.view.dom.style.overflow = "hidden";
    }
    resize(width, height) {
        if (width != this.width)
            this.node.style.width = String(width) + "px";
        if (height != this.height)
            this.node.style.height = String(height) + "px";
        this.width = width;
        this.height = height;
    }
    setSide(side) {
        this.view.dom.style.flexDirection =
            side === "left" ? "row-reverse" : "row";
    }
    setBackground(bg) {
        this.node.style.background = bg;
    }
}

/// The default maximum length of a `TreeBuffer` node.
const DefaultBufferLength = 1024;
let nextPropID = 0;
const CachedNode = new WeakMap();
/// Each [node type](#tree.NodeType) can have metadata associated with
/// it in props. Instances of this class represent prop names.
class NodeProp {
    /// Create a new node prop type. You can optionally pass a
    /// `deserialize` function.
    constructor({ deserialize } = {}) {
        this.id = nextPropID++;
        this.deserialize = deserialize || (() => {
            throw new Error("This node type doesn't define a deserialize function");
        });
    }
    /// Create a string-valued node prop whose deserialize function is
    /// the identity function.
    static string() { return new NodeProp({ deserialize: str => str }); }
    /// Create a number-valued node prop whose deserialize function is
    /// just `Number`.
    static number() { return new NodeProp({ deserialize: Number }); }
    /// Creates a boolean-valued node prop whose deserialize function
    /// returns true for any input.
    static flag() { return new NodeProp({ deserialize: () => true }); }
    /// Store a value for this prop in the given object. This can be
    /// useful when building up a prop object to pass to the
    /// [`NodeType`](#tree.NodeType) constructor. Returns its first
    /// argument.
    set(propObj, value) {
        propObj[this.id] = value;
        return propObj;
    }
    /// This is meant to be used with
    /// [`NodeSet.extend`](#tree.NodeSet.extend) or
    /// [`Parser.withProps`](#lezer.Parser.withProps) to compute prop
    /// values for each node type in the set. Takes a [match
    /// object](#tree.NodeType^match) or function that returns undefined
    /// if the node type doesn't get this prop, and the prop's value if
    /// it does.
    add(match) {
        if (typeof match != "function")
            match = NodeType.match(match);
        return (type) => {
            let result = match(type);
            return result === undefined ? null : [this, result];
        };
    }
}
/// Prop that is used to describe matching delimiters. For opening
/// delimiters, this holds an array of node names (written as a
/// space-separated string when declaring this prop in a grammar)
/// for the node types of closing delimiters that match it.
NodeProp.closedBy = new NodeProp({ deserialize: str => str.split(" ") });
/// The inverse of [`openedBy`](#tree.NodeProp^closedBy). This is
/// attached to closing delimiters, holding an array of node names
/// of types of matching opening delimiters.
NodeProp.openedBy = new NodeProp({ deserialize: str => str.split(" ") });
/// Used to assign node types to groups (for example, all node
/// types that represent an expression could be tagged with an
/// `"Expression"` group).
NodeProp.group = new NodeProp({ deserialize: str => str.split(" ") });
const noProps = Object.create(null);
/// Each node in a syntax tree has a node type associated with it.
class NodeType {
    /// @internal
    constructor(
    /// The name of the node type. Not necessarily unique, but if the
    /// grammar was written properly, different node types with the
    /// same name within a node set should play the same semantic
    /// role.
    name, 
    /// @internal
    props, 
    /// The id of this node in its set. Corresponds to the term ids
    /// used in the parser.
    id, 
    /// @internal
    flags = 0) {
        this.name = name;
        this.props = props;
        this.id = id;
        this.flags = flags;
    }
    static define(spec) {
        let props = spec.props && spec.props.length ? Object.create(null) : noProps;
        let flags = (spec.top ? 1 /* Top */ : 0) | (spec.skipped ? 2 /* Skipped */ : 0) |
            (spec.error ? 4 /* Error */ : 0) | (spec.name == null ? 8 /* Anonymous */ : 0);
        let type = new NodeType(spec.name || "", props, spec.id, flags);
        if (spec.props)
            for (let src of spec.props) {
                if (!Array.isArray(src))
                    src = src(type);
                if (src)
                    src[0].set(props, src[1]);
            }
        return type;
    }
    /// Retrieves a node prop for this type. Will return `undefined` if
    /// the prop isn't present on this node.
    prop(prop) { return this.props[prop.id]; }
    /// True when this is the top node of a grammar.
    get isTop() { return (this.flags & 1 /* Top */) > 0; }
    /// True when this node is produced by a skip rule.
    get isSkipped() { return (this.flags & 2 /* Skipped */) > 0; }
    /// Indicates whether this is an error node.
    get isError() { return (this.flags & 4 /* Error */) > 0; }
    /// When true, this node type doesn't correspond to a user-declared
    /// named node, for example because it is used to cache repetition.
    get isAnonymous() { return (this.flags & 8 /* Anonymous */) > 0; }
    /// Returns true when this node's name or one of its
    /// [groups](#tree.NodeProp^group) matches the given string.
    is(name) {
        if (typeof name == 'string') {
            if (this.name == name)
                return true;
            let group = this.prop(NodeProp.group);
            return group ? group.indexOf(name) > -1 : false;
        }
        return this.id == name;
    }
    /// Create a function from node types to arbitrary values by
    /// specifying an object whose property names are node or
    /// [group](#tree.NodeProp^group) names. Often useful with
    /// [`NodeProp.add`](#tree.NodeProp.add). You can put multiple
    /// names, separated by spaces, in a single property name to map
    /// multiple node names to a single value.
    static match(map) {
        let direct = Object.create(null);
        for (let prop in map)
            for (let name of prop.split(" "))
                direct[name] = map[prop];
        return (node) => {
            for (let groups = node.prop(NodeProp.group), i = -1; i < (groups ? groups.length : 0); i++) {
                let found = direct[i < 0 ? node.name : groups[i]];
                if (found)
                    return found;
            }
        };
    }
}
/// An empty dummy node type to use when no actual type is available.
NodeType.none = new NodeType("", Object.create(null), 0, 8 /* Anonymous */);
/// A piece of syntax tree. There are two ways to approach these
/// trees: the way they are actually stored in memory, and the
/// convenient way.
///
/// Syntax trees are stored as a tree of `Tree` and `TreeBuffer`
/// objects. By packing detail information into `TreeBuffer` leaf
/// nodes, the representation is made a lot more memory-efficient.
///
/// However, when you want to actually work with tree nodes, this
/// representation is very awkward, so most client code will want to
/// use the `TreeCursor` interface instead, which provides a view on
/// some part of this data structure, and can be used to move around
/// to adjacent nodes.
class Tree {
    /// Construct a new tree. You usually want to go through
    /// [`Tree.build`](#tree.Tree^build) instead.
    constructor(type, 
    /// The tree's child nodes. Children small enough to fit in a
    /// `TreeBuffer will be represented as such, other children can be
    /// further `Tree` instances with their own internal structure.
    children, 
    /// The positions (offsets relative to the start of this tree) of
    /// the children.
    positions, 
    /// The total length of this tree
    length) {
        this.type = type;
        this.children = children;
        this.positions = positions;
        this.length = length;
    }
    /// @internal
    toString() {
        let children = this.children.map(c => c.toString()).join();
        return !this.type.name ? children :
            (/\W/.test(this.type.name) && !this.type.isError ? JSON.stringify(this.type.name) : this.type.name) +
                (children.length ? "(" + children + ")" : "");
    }
    /// Get a [tree cursor](#tree.TreeCursor) rooted at this tree. When
    /// `pos` is given, the cursor is [moved](#tree.TreeCursor.moveTo)
    /// to the given position and side.
    cursor(pos, side = 0) {
        let scope = (pos != null && CachedNode.get(this)) || this.topNode;
        let cursor = new TreeCursor(scope);
        if (pos != null) {
            cursor.moveTo(pos, side);
            CachedNode.set(this, cursor._tree);
        }
        return cursor;
    }
    /// Get a [tree cursor](#tree.TreeCursor) that, unlike regular
    /// cursors, doesn't skip [anonymous](#tree.NodeType.isAnonymous)
    /// nodes.
    fullCursor() {
        return new TreeCursor(this.topNode, true);
    }
    /// Get a [syntax node](#tree.SyntaxNode) object for the top of the
    /// tree.
    get topNode() {
        return new TreeNode(this, 0, 0, null);
    }
    /// Get the [syntax node](#tree.SyntaxNode) at the given position.
    /// If `side` is -1, this will move into nodes that end at the
    /// position. If 1, it'll move into nodes that start at the
    /// position. With 0, it'll only enter nodes that cover the position
    /// from both sides.
    resolve(pos, side = 0) {
        return this.cursor(pos, side).node;
    }
    /// Iterate over the tree and its children, calling `enter` for any
    /// node that touches the `from`/`to` region (if given) before
    /// running over such a node's children, and `leave` (if given) when
    /// leaving the node. When `enter` returns `false`, the given node
    /// will not have its children iterated over (or `leave` called).
    iterate(spec) {
        let { enter, leave, from = 0, to = this.length } = spec;
        for (let c = this.cursor();;) {
            let mustLeave = false;
            if (c.from <= to && c.to >= from && (c.type.isAnonymous || enter(c.type, c.from, c.to) !== false)) {
                if (c.firstChild())
                    continue;
                if (!c.type.isAnonymous)
                    mustLeave = true;
            }
            for (;;) {
                if (mustLeave && leave)
                    leave(c.type, c.from, c.to);
                mustLeave = c.type.isAnonymous;
                if (c.nextSibling())
                    break;
                if (!c.parent())
                    return;
                mustLeave = true;
            }
        }
    }
    /// Balance the direct children of this tree.
    balance(maxBufferLength = DefaultBufferLength) {
        return this.children.length <= BalanceBranchFactor ? this
            : balanceRange(this.type, NodeType.none, this.children, this.positions, 0, this.children.length, 0, maxBufferLength, this.length, 0);
    }
    /// Build a tree from a postfix-ordered buffer of node information,
    /// or a cursor over such a buffer.
    static build(data) { return buildTree(data); }
}
/// The empty tree
Tree.empty = new Tree(NodeType.none, [], [], 0);
// For trees that need a context hash attached, we're using this
// kludge which assigns an extra property directly after
// initialization (creating a single new object shape).
function withHash(tree, hash) {
    if (hash)
        tree.contextHash = hash;
    return tree;
}
/// Tree buffers contain (type, start, end, endIndex) quads for each
/// node. In such a buffer, nodes are stored in prefix order (parents
/// before children, with the endIndex of the parent indicating which
/// children belong to it)
class TreeBuffer {
    /// Create a tree buffer @internal
    constructor(
    /// @internal
    buffer, 
    // The total length of the group of nodes in the buffer.
    length, 
    /// @internal
    set, type = NodeType.none) {
        this.buffer = buffer;
        this.length = length;
        this.set = set;
        this.type = type;
    }
    /// @internal
    toString() {
        let result = [];
        for (let index = 0; index < this.buffer.length;) {
            result.push(this.childString(index));
            index = this.buffer[index + 3];
        }
        return result.join(",");
    }
    /// @internal
    childString(index) {
        let id = this.buffer[index], endIndex = this.buffer[index + 3];
        let type = this.set.types[id], result = type.name;
        if (/\W/.test(result) && !type.isError)
            result = JSON.stringify(result);
        index += 4;
        if (endIndex == index)
            return result;
        let children = [];
        while (index < endIndex) {
            children.push(this.childString(index));
            index = this.buffer[index + 3];
        }
        return result + "(" + children.join(",") + ")";
    }
    /// @internal
    findChild(startIndex, endIndex, dir, after) {
        let { buffer } = this, pick = -1;
        for (let i = startIndex; i != endIndex; i = buffer[i + 3]) {
            if (after != -100000000 /* None */) {
                let start = buffer[i + 1], end = buffer[i + 2];
                if (dir > 0) {
                    if (end > after)
                        pick = i;
                    if (end > after)
                        break;
                }
                else {
                    if (start < after)
                        pick = i;
                    if (end >= after)
                        break;
                }
            }
            else {
                pick = i;
                if (dir > 0)
                    break;
            }
        }
        return pick;
    }
}
class TreeNode {
    constructor(node, from, index, _parent) {
        this.node = node;
        this.from = from;
        this.index = index;
        this._parent = _parent;
    }
    get type() { return this.node.type; }
    get name() { return this.node.type.name; }
    get to() { return this.from + this.node.length; }
    nextChild(i, dir, after, full = false) {
        for (let parent = this;;) {
            for (let { children, positions } = parent.node, e = dir > 0 ? children.length : -1; i != e; i += dir) {
                let next = children[i], start = positions[i] + parent.from;
                if (after != -100000000 /* None */ && (dir < 0 ? start >= after : start + next.length <= after))
                    continue;
                if (next instanceof TreeBuffer) {
                    let index = next.findChild(0, next.buffer.length, dir, after == -100000000 /* None */ ? -100000000 /* None */ : after - start);
                    if (index > -1)
                        return new BufferNode(new BufferContext(parent, next, i, start), null, index);
                }
                else if (full || (!next.type.isAnonymous || hasChild(next))) {
                    let inner = new TreeNode(next, start, i, parent);
                    return full || !inner.type.isAnonymous ? inner : inner.nextChild(dir < 0 ? next.children.length - 1 : 0, dir, after);
                }
            }
            if (full || !parent.type.isAnonymous)
                return null;
            i = parent.index + dir;
            parent = parent._parent;
            if (!parent)
                return null;
        }
    }
    get firstChild() { return this.nextChild(0, 1, -100000000 /* None */); }
    get lastChild() { return this.nextChild(this.node.children.length - 1, -1, -100000000 /* None */); }
    childAfter(pos) { return this.nextChild(0, 1, pos); }
    childBefore(pos) { return this.nextChild(this.node.children.length - 1, -1, pos); }
    nextSignificantParent() {
        let val = this;
        while (val.type.isAnonymous && val._parent)
            val = val._parent;
        return val;
    }
    get parent() {
        return this._parent ? this._parent.nextSignificantParent() : null;
    }
    get nextSibling() {
        return this._parent ? this._parent.nextChild(this.index + 1, 1, -1) : null;
    }
    get prevSibling() {
        return this._parent ? this._parent.nextChild(this.index - 1, -1, -1) : null;
    }
    get cursor() { return new TreeCursor(this); }
    resolve(pos, side = 0) {
        return this.cursor.moveTo(pos, side).node;
    }
    getChild(type, before = null, after = null) {
        let r = getChildren(this, type, before, after);
        return r.length ? r[0] : null;
    }
    getChildren(type, before = null, after = null) {
        return getChildren(this, type, before, after);
    }
    /// @internal
    toString() { return this.node.toString(); }
}
function getChildren(node, type, before, after) {
    let cur = node.cursor, result = [];
    if (!cur.firstChild())
        return result;
    if (before != null)
        while (!cur.type.is(before))
            if (!cur.nextSibling())
                return result;
    for (;;) {
        if (after != null && cur.type.is(after))
            return result;
        if (cur.type.is(type))
            result.push(cur.node);
        if (!cur.nextSibling())
            return after == null ? result : [];
    }
}
class BufferContext {
    constructor(parent, buffer, index, start) {
        this.parent = parent;
        this.buffer = buffer;
        this.index = index;
        this.start = start;
    }
}
class BufferNode {
    constructor(context, _parent, index) {
        this.context = context;
        this._parent = _parent;
        this.index = index;
        this.type = context.buffer.set.types[context.buffer.buffer[index]];
    }
    get name() { return this.type.name; }
    get from() { return this.context.start + this.context.buffer.buffer[this.index + 1]; }
    get to() { return this.context.start + this.context.buffer.buffer[this.index + 2]; }
    child(dir, after) {
        let { buffer } = this.context;
        let index = buffer.findChild(this.index + 4, buffer.buffer[this.index + 3], dir, after == -100000000 /* None */ ? -100000000 /* None */ : after - this.context.start);
        return index < 0 ? null : new BufferNode(this.context, this, index);
    }
    get firstChild() { return this.child(1, -100000000 /* None */); }
    get lastChild() { return this.child(-1, -100000000 /* None */); }
    childAfter(pos) { return this.child(1, pos); }
    childBefore(pos) { return this.child(-1, pos); }
    get parent() {
        return this._parent || this.context.parent.nextSignificantParent();
    }
    externalSibling(dir) {
        return this._parent ? null : this.context.parent.nextChild(this.context.index + dir, dir, -1);
    }
    get nextSibling() {
        let { buffer } = this.context;
        let after = buffer.buffer[this.index + 3];
        if (after < (this._parent ? buffer.buffer[this._parent.index + 3] : buffer.buffer.length))
            return new BufferNode(this.context, this._parent, after);
        return this.externalSibling(1);
    }
    get prevSibling() {
        let { buffer } = this.context;
        let parentStart = this._parent ? this._parent.index + 4 : 0;
        if (this.index == parentStart)
            return this.externalSibling(-1);
        return new BufferNode(this.context, this._parent, buffer.findChild(parentStart, this.index, -1, -100000000 /* None */));
    }
    get cursor() { return new TreeCursor(this); }
    resolve(pos, side = 0) {
        return this.cursor.moveTo(pos, side).node;
    }
    /// @internal
    toString() { return this.context.buffer.childString(this.index); }
    getChild(type, before = null, after = null) {
        let r = getChildren(this, type, before, after);
        return r.length ? r[0] : null;
    }
    getChildren(type, before = null, after = null) {
        return getChildren(this, type, before, after);
    }
}
/// A tree cursor object focuses on a given node in a syntax tree, and
/// allows you to move to adjacent nodes.
class TreeCursor {
    /// @internal
    constructor(node, full = false) {
        this.full = full;
        this.buffer = null;
        this.stack = [];
        this.index = 0;
        this.bufferNode = null;
        if (node instanceof TreeNode) {
            this.yieldNode(node);
        }
        else {
            this._tree = node.context.parent;
            this.buffer = node.context;
            for (let n = node._parent; n; n = n._parent)
                this.stack.unshift(n.index);
            this.bufferNode = node;
            this.yieldBuf(node.index);
        }
    }
    /// Shorthand for `.type.name`.
    get name() { return this.type.name; }
    yieldNode(node) {
        if (!node)
            return false;
        this._tree = node;
        this.type = node.type;
        this.from = node.from;
        this.to = node.to;
        return true;
    }
    yieldBuf(index, type) {
        this.index = index;
        let { start, buffer } = this.buffer;
        this.type = type || buffer.set.types[buffer.buffer[index]];
        this.from = start + buffer.buffer[index + 1];
        this.to = start + buffer.buffer[index + 2];
        return true;
    }
    yield(node) {
        if (!node)
            return false;
        if (node instanceof TreeNode) {
            this.buffer = null;
            return this.yieldNode(node);
        }
        this.buffer = node.context;
        return this.yieldBuf(node.index, node.type);
    }
    /// @internal
    toString() {
        return this.buffer ? this.buffer.buffer.childString(this.index) : this._tree.toString();
    }
    /// @internal
    enter(dir, after) {
        if (!this.buffer)
            return this.yield(this._tree.nextChild(dir < 0 ? this._tree.node.children.length - 1 : 0, dir, after, this.full));
        let { buffer } = this.buffer;
        let index = buffer.findChild(this.index + 4, buffer.buffer[this.index + 3], dir, after == -100000000 /* None */ ? -100000000 /* None */ : after - this.buffer.start);
        if (index < 0)
            return false;
        this.stack.push(this.index);
        return this.yieldBuf(index);
    }
    /// Move the cursor to this node's first child. When this returns
    /// false, the node has no child, and the cursor has not been moved.
    firstChild() { return this.enter(1, -100000000 /* None */); }
    /// Move the cursor to this node's last child.
    lastChild() { return this.enter(-1, -100000000 /* None */); }
    /// Move the cursor to the first child that starts at or after `pos`.
    childAfter(pos) { return this.enter(1, pos); }
    /// Move to the last child that ends at or before `pos`.
    childBefore(pos) { return this.enter(-1, pos); }
    /// Move the node's parent node, if this isn't the top node.
    parent() {
        if (!this.buffer)
            return this.yieldNode(this.full ? this._tree._parent : this._tree.parent);
        if (this.stack.length)
            return this.yieldBuf(this.stack.pop());
        let parent = this.full ? this.buffer.parent : this.buffer.parent.nextSignificantParent();
        this.buffer = null;
        return this.yieldNode(parent);
    }
    /// @internal
    sibling(dir) {
        if (!this.buffer)
            return !this._tree._parent ? false
                : this.yield(this._tree._parent.nextChild(this._tree.index + dir, dir, -100000000 /* None */, this.full));
        let { buffer } = this.buffer, d = this.stack.length - 1;
        if (dir < 0) {
            let parentStart = d < 0 ? 0 : this.stack[d] + 4;
            if (this.index != parentStart)
                return this.yieldBuf(buffer.findChild(parentStart, this.index, -1, -100000000 /* None */));
        }
        else {
            let after = buffer.buffer[this.index + 3];
            if (after < (d < 0 ? buffer.buffer.length : buffer.buffer[this.stack[d] + 3]))
                return this.yieldBuf(after);
        }
        return d < 0 ? this.yield(this.buffer.parent.nextChild(this.buffer.index + dir, dir, -100000000 /* None */, this.full)) : false;
    }
    /// Move to this node's next sibling, if any.
    nextSibling() { return this.sibling(1); }
    /// Move to this node's previous sibling, if any.
    prevSibling() { return this.sibling(-1); }
    atLastNode(dir) {
        let index, parent, { buffer } = this;
        if (buffer) {
            if (dir > 0) {
                if (this.index < buffer.buffer.buffer.length)
                    return false;
            }
            else {
                for (let i = 0; i < this.index; i++)
                    if (buffer.buffer.buffer[i + 3] < this.index)
                        return false;
            }
            ({ index, parent } = buffer);
        }
        else {
            ({ index, _parent: parent } = this._tree);
        }
        for (; parent; { index, _parent: parent } = parent) {
            for (let i = index + dir, e = dir < 0 ? -1 : parent.node.children.length; i != e; i += dir) {
                let child = parent.node.children[i];
                if (this.full || !child.type.isAnonymous || child instanceof TreeBuffer || hasChild(child))
                    return false;
            }
        }
        return true;
    }
    move(dir) {
        if (this.enter(dir, -100000000 /* None */))
            return true;
        for (;;) {
            if (this.sibling(dir))
                return true;
            if (this.atLastNode(dir) || !this.parent())
                return false;
        }
    }
    /// Move to the next node in a
    /// [pre-order](https://en.wikipedia.org/wiki/Tree_traversal#Pre-order_(NLR))
    /// traversal, going from a node to its first child or, if the
    /// current node is empty, its next sibling or the next sibling of
    /// the first parent node that has one.
    next() { return this.move(1); }
    /// Move to the next node in a last-to-first pre-order traveral. A
    /// node is followed by ist last child or, if it has none, its
    /// previous sibling or the previous sibling of the first parent
    /// node that has one.
    prev() { return this.move(-1); }
    /// Move the cursor to the innermost node that covers `pos`. If
    /// `side` is -1, it will enter nodes that end at `pos`. If it is 1,
    /// it will enter nodes that start at `pos`.
    moveTo(pos, side = 0) {
        // Move up to a node that actually holds the position, if possible
        while (this.from == this.to ||
            (side < 1 ? this.from >= pos : this.from > pos) ||
            (side > -1 ? this.to <= pos : this.to < pos))
            if (!this.parent())
                break;
        // Then scan down into child nodes as far as possible
        for (;;) {
            if (side < 0 ? !this.childBefore(pos) : !this.childAfter(pos))
                break;
            if (this.from == this.to ||
                (side < 1 ? this.from >= pos : this.from > pos) ||
                (side > -1 ? this.to <= pos : this.to < pos)) {
                this.parent();
                break;
            }
        }
        return this;
    }
    /// Get a [syntax node](#tree.SyntaxNode) at the cursor's current
    /// position.
    get node() {
        if (!this.buffer)
            return this._tree;
        let cache = this.bufferNode, result = null, depth = 0;
        if (cache && cache.context == this.buffer) {
            scan: for (let index = this.index, d = this.stack.length; d >= 0;) {
                for (let c = cache; c; c = c._parent)
                    if (c.index == index) {
                        if (index == this.index)
                            return c;
                        result = c;
                        depth = d + 1;
                        break scan;
                    }
                index = this.stack[--d];
            }
        }
        for (let i = depth; i < this.stack.length; i++)
            result = new BufferNode(this.buffer, result, this.stack[i]);
        return this.bufferNode = new BufferNode(this.buffer, result, this.index);
    }
    /// Get the [tree](#tree.Tree) that represents the current node, if
    /// any. Will return null when the node is in a [tree
    /// buffer](#tree.TreeBuffer).
    get tree() {
        return this.buffer ? null : this._tree.node;
    }
}
function hasChild(tree) {
    return tree.children.some(ch => !ch.type.isAnonymous || ch instanceof TreeBuffer || hasChild(ch));
}
class FlatBufferCursor {
    constructor(buffer, index) {
        this.buffer = buffer;
        this.index = index;
    }
    get id() { return this.buffer[this.index - 4]; }
    get start() { return this.buffer[this.index - 3]; }
    get end() { return this.buffer[this.index - 2]; }
    get size() { return this.buffer[this.index - 1]; }
    get pos() { return this.index; }
    next() { this.index -= 4; }
    fork() { return new FlatBufferCursor(this.buffer, this.index); }
}
const BalanceBranchFactor = 8;
function buildTree(data) {
    var _a;
    let { buffer, nodeSet, topID = 0, maxBufferLength = DefaultBufferLength, reused = [], minRepeatType = nodeSet.types.length } = data;
    let cursor = Array.isArray(buffer) ? new FlatBufferCursor(buffer, buffer.length) : buffer;
    let types = nodeSet.types;
    let contextHash = 0;
    function takeNode(parentStart, minPos, children, positions, inRepeat) {
        let { id, start, end, size } = cursor;
        let startPos = start - parentStart;
        if (size < 0) {
            if (size == -1) { // Reused node
                children.push(reused[id]);
                positions.push(startPos);
            }
            else { // Context change
                contextHash = id;
            }
            cursor.next();
            return;
        }
        let type = types[id], node, buffer;
        if (end - start <= maxBufferLength && (buffer = findBufferSize(cursor.pos - minPos, inRepeat))) {
            // Small enough for a buffer, and no reused nodes inside
            let data = new Uint16Array(buffer.size - buffer.skip);
            let endPos = cursor.pos - buffer.size, index = data.length;
            while (cursor.pos > endPos)
                index = copyToBuffer(buffer.start, data, index, inRepeat);
            node = new TreeBuffer(data, end - buffer.start, nodeSet, inRepeat < 0 ? NodeType.none : types[inRepeat]);
            startPos = buffer.start - parentStart;
        }
        else { // Make it a node
            let endPos = cursor.pos - size;
            cursor.next();
            let localChildren = [], localPositions = [];
            let localInRepeat = id >= minRepeatType ? id : -1;
            while (cursor.pos > endPos) {
                if (cursor.id == localInRepeat)
                    cursor.next();
                else
                    takeNode(start, endPos, localChildren, localPositions, localInRepeat);
            }
            localChildren.reverse();
            localPositions.reverse();
            if (localInRepeat > -1 && localChildren.length > BalanceBranchFactor)
                node = balanceRange(type, type, localChildren, localPositions, 0, localChildren.length, 0, maxBufferLength, end - start, contextHash);
            else
                node = withHash(new Tree(type, localChildren, localPositions, end - start), contextHash);
        }
        children.push(node);
        positions.push(startPos);
    }
    function findBufferSize(maxSize, inRepeat) {
        // Scan through the buffer to find previous siblings that fit
        // together in a TreeBuffer, and don't contain any reused nodes
        // (which can't be stored in a buffer).
        // If `inRepeat` is > -1, ignore node boundaries of that type for
        // nesting, but make sure the end falls either at the start
        // (`maxSize`) or before such a node.
        let fork = cursor.fork();
        let size = 0, start = 0, skip = 0, minStart = fork.end - maxBufferLength;
        let result = { size: 0, start: 0, skip: 0 };
        scan: for (let minPos = fork.pos - maxSize; fork.pos > minPos;) {
            // Pretend nested repeat nodes of the same type don't exist
            if (fork.id == inRepeat) {
                // Except that we store the current state as a valid return
                // value.
                result.size = size;
                result.start = start;
                result.skip = skip;
                skip += 4;
                size += 4;
                fork.next();
                continue;
            }
            let nodeSize = fork.size, startPos = fork.pos - nodeSize;
            if (nodeSize < 0 || startPos < minPos || fork.start < minStart)
                break;
            let localSkipped = fork.id >= minRepeatType ? 4 : 0;
            let nodeStart = fork.start;
            fork.next();
            while (fork.pos > startPos) {
                if (fork.size < 0)
                    break scan;
                if (fork.id >= minRepeatType)
                    localSkipped += 4;
                fork.next();
            }
            start = nodeStart;
            size += nodeSize;
            skip += localSkipped;
        }
        if (inRepeat < 0 || size == maxSize) {
            result.size = size;
            result.start = start;
            result.skip = skip;
        }
        return result.size > 4 ? result : undefined;
    }
    function copyToBuffer(bufferStart, buffer, index, inRepeat) {
        let { id, start, end, size } = cursor;
        cursor.next();
        if (id == inRepeat)
            return index;
        let startIndex = index;
        if (size > 4) {
            let endPos = cursor.pos - (size - 4);
            while (cursor.pos > endPos)
                index = copyToBuffer(bufferStart, buffer, index, inRepeat);
        }
        if (id < minRepeatType) { // Don't copy repeat nodes into buffers
            buffer[--index] = startIndex;
            buffer[--index] = end - bufferStart;
            buffer[--index] = start - bufferStart;
            buffer[--index] = id;
        }
        return index;
    }
    let children = [], positions = [];
    while (cursor.pos > 0)
        takeNode(data.start || 0, 0, children, positions, -1);
    let length = (_a = data.length) !== null && _a !== void 0 ? _a : (children.length ? positions[0] + children[0].length : 0);
    return new Tree(types[topID], children.reverse(), positions.reverse(), length);
}
function balanceRange(outerType, innerType, children, positions, from, to, start, maxBufferLength, length, contextHash) {
    let localChildren = [], localPositions = [];
    if (length <= maxBufferLength) {
        for (let i = from; i < to; i++) {
            localChildren.push(children[i]);
            localPositions.push(positions[i] - start);
        }
    }
    else {
        let maxChild = Math.max(maxBufferLength, Math.ceil(length * 1.5 / BalanceBranchFactor));
        for (let i = from; i < to;) {
            let groupFrom = i, groupStart = positions[i];
            i++;
            for (; i < to; i++) {
                let nextEnd = positions[i] + children[i].length;
                if (nextEnd - groupStart > maxChild)
                    break;
            }
            if (i == groupFrom + 1) {
                let only = children[groupFrom];
                if (only instanceof Tree && only.type == innerType && only.length > maxChild << 1) { // Too big, collapse
                    for (let j = 0; j < only.children.length; j++) {
                        localChildren.push(only.children[j]);
                        localPositions.push(only.positions[j] + groupStart - start);
                    }
                    continue;
                }
                localChildren.push(only);
            }
            else if (i == groupFrom + 1) {
                localChildren.push(children[groupFrom]);
            }
            else {
                let inner = balanceRange(innerType, innerType, children, positions, groupFrom, i, groupStart, maxBufferLength, positions[i - 1] + children[i - 1].length - groupStart, contextHash);
                if (innerType != NodeType.none && !containsType(inner.children, innerType))
                    inner = withHash(new Tree(NodeType.none, inner.children, inner.positions, inner.length), contextHash);
                localChildren.push(inner);
            }
            localPositions.push(groupStart - start);
        }
    }
    return withHash(new Tree(outerType, localChildren, localPositions, length), contextHash);
}
function containsType(nodes, type) {
    for (let elt of nodes)
        if (elt.type == type)
            return true;
    return false;
}

var Recover;
(function (Recover) {
    Recover[Recover["Token"] = 200] = "Token";
    Recover[Recover["Reduce"] = 100] = "Reduce";
    Recover[Recover["MaxNext"] = 4] = "MaxNext";
    Recover[Recover["MaxInsertStackDepth"] = 300] = "MaxInsertStackDepth";
    Recover[Recover["DampenInsertStackDepth"] = 120] = "DampenInsertStackDepth";
})(Recover || (Recover = {}));

// FIXME find some way to reduce recovery work done when the input
// doesn't match the grammar at all.
// Environment variable used to control console output
typeof process != "undefined" && /\bparse\b/.test(process.env.LOG);
var Rec;
(function (Rec) {
    Rec[Rec["Distance"] = 5] = "Distance";
    Rec[Rec["MaxRemainingPerStep"] = 3] = "MaxRemainingPerStep";
    Rec[Rec["MinBufferLengthPrune"] = 200] = "MinBufferLengthPrune";
    Rec[Rec["ForceReduceLimit"] = 10] = "ForceReduceLimit";
})(Rec || (Rec = {}));

const translateDiagnostics = (input = {
    checkResult: { checkItems: [], rawSQL: "" },
}, view) => {
    const { checkResult: { checkItems = [] }, } = input;
    const found = [];
    const maxLines = view.state.doc.lines;
    checkItems.forEach((item) => {
        const { error, errorCode, lineOffsets } = item;
        lineOffsets.forEach((lineOffset) => {
            let { StartLine, EndLine, StartOffset, EndOffset } = lineOffset;
            if (StartLine < 0) {
                StartLine = 0;
            }
            if (EndLine < 0) {
                EndLine = 0;
            }
            if (StartOffset < 0) {
                StartOffset = 0;
            }
            if (EndOffset < 0) {
                EndOffset = 0;
            }
            let from, to;
            if (maxLines >= EndLine) {
                const { from: startFrom, to: startTo } = view.state.doc.line(StartLine + 1);
                const { from: endFrom, to: endTo } = view.state.doc.line(EndLine + 1);
                if (startFrom === endFrom && startFrom === endTo) {
                    // same line
                    from = startFrom + StartOffset;
                    to = startTo + EndOffset;
                }
                else {
                    from = startFrom + StartOffset;
                    to = endFrom + EndOffset;
                }
                found.push({
                    from,
                    to,
                    severity: errorCode == -1 ? "error" : "warning",
                    message: error,
                });
            }
        });
    });
    return found.sort((a, b) => {
        if (a.from < b.from) {
            return -1;
        }
        if (a.from > b.from) {
            return 1;
        }
        return 0;
    });
};
const isSameTextAccordingToDoc = (editor, value = "") => {
    const doc = editor.state.doc.toJSON();
    const nextDocText = value.split(editor.state.lineBreak);
    if (doc.length !== nextDocText.length) {
        return false;
    }
    let i = doc.length - 1;
    while (i >= 0) {
        if (doc[i] !== nextDocText[i]) {
            return false;
        }
        i--;
    }
    return true;
};
function getStyle(element, attr) {
    var _a;
    return ((_a = window === null || window === void 0 ? void 0 : window.getComputedStyle(element, null)[attr]) !== null && _a !== void 0 ? _a : "");
}

// const SpecialChars = [
//   "(",
//   ")",
//   "{",
//   "}",
//   "[",
//   "]",
//   ",",
//   ";",
//   "-",
//   "*",
// ]
const LineHeight = 1;
const WordGap = 5;
class Drawer {
    constructor(view, ctx, cache) {
        ///
        this.tree = Tree.empty;
        this.view = view;
        this.ctx = ctx;
        this.cache = cache;
    }
    /// 绘制内容
    draw(view) {
        const doc = view.state.doc;
        const totalLines = doc.lines;
        const textIter = doc.iter(1);
        while (!textIter.done) {
            console.log(textIter);
            textIter.next();
        }
        let index = 1;
        while (index <= totalLines) {
            this.drawLine(doc.line(index));
            index++;
        }
    }
    /// 处理一行的字符
    drawLine(line) {
        const { text, from } = line;
        let offset = 2;
        if (/^\s.*/.test(text)) {
            offset = /\s+/.exec(text)[0].length + 10;
        }
        const tokens = text.split(/\s/);
        // Reset Prev Token
        this.posX = null;
        tokens.forEach((token, index) => {
            const pos = from + line.text.indexOf(token) + 1;
            let color;
            const { node } = this.view.domAtPos(pos);
            const cacheToken = this.cache.get(token + String(line.number));
            if (!cacheToken ||
                cacheToken.text !== token ||
                cacheToken.pos !== pos ||
                token !== "") {
                this.cache.set(token + String(line.number), {
                    text: token,
                    pos,
                });
                color = getStyle(node.parentElement, "color");
                this.drawToken(line, {
                    color,
                    text: token,
                }, offset);
                // }
            }
        });
    }
    /// 处理单个字符
    drawToken(line, token, offset) {
        const color = token.color || "white";
        // let length: number, x: number, y: number
        let x = offset; // token渲染的左偏移
        const y = line.number + line.number * LineHeight; // token渲染的上偏移
        const length = token.text.trimStart().length * 5; // token的长度，原始长度太小，增加五倍的长度
        if (this.posX) {
            x = this.posX + WordGap;
        }
        this.drawRect(x, y, color, length, LineHeight);
        this.posX = x + length;
    }
    /// 绘制色块
    drawRect(x, y, color, width, height) {
        this.ctx.fillStyle = color;
        this.ctx.fillRect(x, y, width, height);
    }
    /// 清理canvas内容
    clear(x, y, w, h) {
        this.ctx.clearRect(x, y, w, h);
    }
}

class TokenCache {
    constructor() {
        this.tokens = new Map();
    }
    get(key) {
        return this.tokens.get(key);
    }
    set(key, value) {
        this.tokens.set(key, value);
    }
    clear() { }
}

class Minimap {
    constructor(view) {
        this.view = view;
        this.cache = new TokenCache();
        this.minimap = new MinimapElement(view);
        this.canvas = new CanvasElement(this.minimap.node);
        this.viewbox = new Viewbox(this.minimap.node);
        this.drawer = new Drawer(view, this.canvas.ctx, this.cache);
    }
    resize(width, height) {
        this.minimap.resize(width, height);
        this.canvas.resize(width, height);
        this.viewbox.resize(width, height);
    }
    setBackground(bg) {
        this.minimap.setBackground(bg);
    }
    // move .cm-minimap under .cm-editor
    detach() {
        const panels = this.view.dom.querySelector(".cm-panels");
        this.view.dom.removeChild(panels);
        return this;
    }
    attach() {
        this.view.dom.appendChild(this.minimap.node);
        return this;
    }
    render(view) {
        if (!view) {
            view = this.view;
        }
        const cursorLine = view.state.doc.lineAt(view.state.selection.main.head);
        // clear canvas area below input line
        this.drawer.clear(0, cursorLine.number * 2, 80 * 5, view.dom.clientHeight);
        this.drawer.draw(view ? view : this.view);
    }
}

const minimapConfig = Facet.define({
    combine(configs) {
        let minimap, minimapWidth, minimapSide;
        for (const config of configs) {
            minimap = config.minimap;
            minimapWidth = config.minimapWidth;
            minimapSide = config.minimapSide;
        }
        return {
            minimap: minimap,
            minimapSide: minimapSide || "left",
            minimapWidth: minimapWidth || 80,
        };
    },
});
const createMinimapPanel = (view) => {
    const config = view.state.facet(minimapConfig);
    const minimapPanel = new Minimap(view);
    return {
        top: true,
        dom: minimapPanel.minimap.node,
        update(update) {
            if (update.docChanged) {
                minimapPanel.render(update.view);
            }
        },
        mount() {
            view.requestMeasure({
                write() {
                    // FIXME 读取minimap是否是打开状态，是的话，才进行read方法里面的操作
                },
                read(view) {
                    minimapPanel.resize(config.minimapWidth, view.dom.clientHeight);
                    minimapPanel.detach().attach();
                    minimapPanel.minimap.setSide(config.minimapSide);
                    minimapPanel.render();
                },
            });
        },
    };
};
const toggleMinimap = StateEffect.define();
const minimap = StateField.define({
    create() {
        return true;
    },
    update(update, tr) {
        for (const e of tr.effects)
            if (e.is(toggleMinimap))
                update = e.value;
        return update;
    },
    provide(field) {
        return showPanel.from(field, (show) => show ? createMinimapPanel : null);
    },
});
[
    minimap,
    EditorView.theme({
        ".cm-panel": {
            backgroundColor: "#21252b",
            color: "#abb2bf",
        },
    }),
];

function startFormat(view, configs) {
    const str = configs.parser(view.state.doc.toJSON().join(view.state.lineBreak));
    Promise.resolve(str).then((resolveStr) => {
        view.dispatch({
            changes: [
                {
                    from: 0,
                    to: view.state.doc.length,
                    insert: resolveStr,
                },
            ],
        });
    });
}

/// feature flags
const MINIMAP_FLAG = false;

/// editable
const editableCompart = new Compartment();
function reconfigure$3(editable) {
    return editableCompart.reconfigure(EditorView.editable.of(editable));
}
function useEditableProp(value, view) {
    const [compart] = useState(() => editableCompart.of(EditorView.editable.of(value)));
    useEffect(() => {
        view === null || view === void 0 ? void 0 : view.dispatch({
            effects: [reconfigure$3(value)],
        });
    }, [value, view]);
    return compart;
}

/// schema completion refactory
/// [@codemirror/lang-sql]本身带的schema自动补全不支持输入空格的时候展示补全
function walkTree(tree, traveler) {
    tree.iterate({
        enter(type, from, to) {
            var _a;
            if (type.name in traveler) {
                (_a = traveler[type.name]) === null || _a === void 0 ? void 0 : _a.call(traveler, type, from, to);
            }
        },
    });
}
function completionFromSpace() {
    let hitCreateKeyword = false;
    let completions = [];
    return (context) => {
        completions = [];
        const tree = syntaxTree(context.state);
        walkTree(tree, {
            Identifier: (_node, from, to) => {
                const name = context.state.doc.sliceString(from, to);
                if (hitCreateKeyword) {
                    completions.push(name);
                    hitCreateKeyword = false;
                }
            },
            Keyword: (_node, from, to) => {
                const name = context.state.doc.sliceString(from, to);
                hitCreateKeyword = name.toLowerCase() === "table";
            },
        });
        return {
            from: context.pos,
            to: context.pos + 1,
            options: completions.map((complete) => ({ label: complete })),
            span: /\w*/,
        };
    };
}
function spaceCompletion(dialect) {
    return [
        keymap.of([
            {
                key: "Space",
                run: startCompletion,
            },
        ]),
        dialect.language.data.of({
            autocomplete: completionFromSpace(),
        }),
    ];
}

function sql(config = {}) {
    const configs = Object.assign({}, config);
    if (!configs.schema) {
        configs.schema = {};
    }
    const lang = configs.dialect || StandardSQL;
    return new LanguageSupport(lang.language, [
        spaceCompletion(lang),
        schemaCompletion(configs),
        keywordCompletion(lang, !!configs.upperCaseKeywords),
    ]);
}

/// language langOptions
const LANGUAGES = {
    javascript,
    sql,
    json,
};
const languageCompart = new Compartment();
function reconfigure$2(language, langOptions) {
    return languageCompart.reconfigure(LANGUAGES[language](
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    //@ts-ignore
    langOptions));
}
function useLanguageProp(language, langOptions, view) {
    const prevLanguage = useRef({
        language,
        langOptions,
    });
    const [compart] = useState(() => languageCompart.of(LANGUAGES[language](
    /// 有些language方法并不需要参数，所以这里会有TS的一场类型错误
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    langOptions)));
    useEffect(() => {
        if (view &&
            (prevLanguage.current.language !== language ||
                !dequal(langOptions, prevLanguage.current.langOptions))) {
            view.dispatch({
                effects: [reconfigure$2(language, langOptions)],
            });
        }
    }, [language, view, langOptions]);
    return compart;
}

// Using https://github.com/one-dark/vscode-one-dark-theme/ as reference for the colors
const chalky$1 = "#e5c07b", coral$1 = "#e06c75", cyan$1 = "#56b6c2", invalid$1 = "#ffffff", ivory$1 = "#abb2bf", stone$1 = "#7d8799", // Brightened compared to original to increase contrast
malibu$1 = "#61afef", sage$1 = "#98c379", whiskey$1 = "#d19a66", violet$1 = "#c678dd", darkBackground$1 = "#21252b", highlightBackground$1 = "#2c313a", background$1 = "#282c34", selection$1 = "#3E4451", cursor$1 = "#528bff";
/// The editor theme styles for One Dark.
const oneDarkTheme$1 = EditorView.theme({
    "&": {
        color: ivory$1,
        backgroundColor: background$1,
        "& ::selection": { backgroundColor: selection$1 },
        caretColor: cursor$1,
        height: "100%",
    },
    "&.cm-focused .cm-cursor": { borderLeftColor: cursor$1 },
    "&.cm-focused .cm-selectionBackground, .cm-selectionBackground": {
        backgroundColor: selection$1,
    },
    ".cm-panels": { backgroundColor: darkBackground$1, color: ivory$1 },
    ".cm-panels.cm-panels-top": { borderBottom: "2px solid black" },
    ".cm-panels.cm-panels-bottom": { borderTop: "2px solid black" },
    ".cm-searchMatch": {
        backgroundColor: "#72a1ff59",
        outline: "1px solid #457dff",
    },
    ".cm-searchMatch.cm-searchMatch-selected": {
        backgroundColor: "#6199ff2f",
    },
    ".cm-activeLine": { backgroundColor: highlightBackground$1 },
    ".cm-selectionMatch": { backgroundColor: "#aafe661a" },
    ".cm-matchingBracket, .cm-nonmatchingBracket": {
        backgroundColor: "#bad0f847",
        outline: "1px solid #515a6b",
    },
    ".cm-gutters": {
        backgroundColor: background$1,
        color: stone$1,
        border: "none",
    },
    ".cm-lineNumbers .cm-gutterElement": { color: "inherit" },
    ".cm-foldPlaceholder": {
        backgroundColor: "transparent",
        border: "none",
        color: "#ddd",
    },
    ".cm-tooltip": {
        border: "1px solid #181a1f",
        backgroundColor: darkBackground$1,
    },
    ".cm-tooltip-autocomplete": {
        "& > ul > li[aria-selected]": {
            backgroundColor: highlightBackground$1,
            color: ivory$1,
        },
        "& > ul > li > cm-completionIcon": {
            paddingRight: "2em",
        },
    },
}, { dark: true });
/// The highlighting style for code in the One Dark theme.
const oneDarkHighlightStyle$1 = HighlightStyle.define([
    { tag: tags.keyword, color: violet$1 },
    {
        tag: [
            tags.name,
            tags.deleted,
            tags.character,
            tags.propertyName,
            tags.macroName,
        ],
        color: coral$1,
    },
    {
        tag: [tags.function(tags.variableName), tags.labelName],
        color: malibu$1,
    },
    {
        tag: [
            tags.color,
            tags.constant(tags.name),
            tags.standard(tags.name),
        ],
        color: whiskey$1,
    },
    { tag: [tags.definition(tags.name), tags.separator], color: ivory$1 },
    {
        tag: [
            tags.typeName,
            tags.className,
            tags.number,
            tags.changed,
            tags.annotation,
            tags.modifier,
            tags.self,
            tags.namespace,
        ],
        color: chalky$1,
    },
    {
        tag: [
            tags.operator,
            tags.operatorKeyword,
            tags.url,
            tags.escape,
            tags.regexp,
            tags.link,
            tags.special(tags.string),
        ],
        color: cyan$1,
    },
    { tag: [tags.meta, tags.comment], color: stone$1 },
    { tag: tags.strong, fontWeight: "bold" },
    { tag: tags.emphasis, fontStyle: "italic" },
    { tag: tags.link, color: stone$1, textDecoration: "underline" },
    { tag: tags.heading, fontWeight: "bold", color: coral$1 },
    {
        tag: [tags.atom, tags.bool, tags.special(tags.variableName)],
        color: whiskey$1,
    },
    {
        tag: [tags.processingInstruction, tags.string, tags.inserted],
        color: sage$1,
    },
    { tag: tags.invalid, color: invalid$1 },
]);
/// Extension to enable the One Dark theme (both the editor theme and
/// the highlight style).
const oneDark$1 = [oneDarkTheme$1, oneDarkHighlightStyle$1];

// Using https://github.com/one-dark/vscode-one-dark-theme/ as reference for the colors
const chalky = "#e5c07b", coral = "#e06c75", cyan = "#56b6c2", invalid = "#ffffff", ivory = "#abb2bf", stone = "#7d8799", // Brightened compared to original to increase contrast
malibu = "#61afef", sage = "#98c379", whiskey = "#d19a66", violet = "#c678dd", darkBackground = "#21252b", highlightBackground = "#e2e2e2", background = "#ffffff", selection = "#3E4451", cursor = "#528bff";
/// The editor theme styles for One Dark.
const oneDarkTheme = EditorView.theme({
    "&": {
        color: ivory,
        backgroundColor: background,
        "& ::selection": { backgroundColor: selection },
        caretColor: cursor,
        height: '100%'
    },
    "&.cm-focused .cm-cursor": { borderLeftColor: cursor },
    "&.cm-focused .cm-selectionBackground, .cm-selectionBackground": {
        backgroundColor: selection,
    },
    ".cm-panels": { backgroundColor: darkBackground, color: ivory },
    ".cm-panels.cm-panels-top": { borderBottom: "2px solid black" },
    ".cm-panels.cm-panels-bottom": { borderTop: "2px solid black" },
    ".cm-searchMatch": {
        backgroundColor: "#72a1ff59",
        outline: "1px solid #457dff",
    },
    ".cm-searchMatch.cm-searchMatch-selected": {
        backgroundColor: "#6199ff2f",
    },
    ".cm-activeLine": { backgroundColor: highlightBackground },
    ".cm-selectionMatch": { backgroundColor: "#aafe661a" },
    ".cm-matchingBracket, .cm-nonmatchingBracket": {
        backgroundColor: "#bad0f847",
        outline: "1px solid #515a6b",
    },
    ".cm-gutters": {
        backgroundColor: background,
        color: stone,
        border: "none",
    },
    ".cm-lineNumbers .cm-gutterElement": { color: "inherit" },
    ".cm-foldPlaceholder": {
        backgroundColor: "transparent",
        border: "none",
        color: "#ddd",
    },
    ".cm-tooltip": {
        border: "1px solid #181a1f",
        backgroundColor: darkBackground,
    },
    ".cm-tooltip-autocomplete": {
        "& > ul > li[aria-selected]": {
            backgroundColor: highlightBackground,
            color: ivory,
        },
        "& > ul > li > cm-completionIcon": {
            paddingRight: "2em",
        },
    },
}, { dark: true });
/// The highlighting style for code in the One Dark theme.
const oneDarkHighlightStyle = HighlightStyle.define([
    { tag: tags.keyword, color: violet },
    {
        tag: [
            tags.name,
            tags.deleted,
            tags.character,
            tags.propertyName,
            tags.macroName,
        ],
        color: coral,
    },
    {
        tag: [tags.function(tags.variableName), tags.labelName],
        color: malibu,
    },
    {
        tag: [
            tags.color,
            tags.constant(tags.name),
            tags.standard(tags.name),
        ],
        color: whiskey,
    },
    { tag: [tags.definition(tags.name), tags.separator], color: ivory },
    {
        tag: [
            tags.typeName,
            tags.className,
            tags.number,
            tags.changed,
            tags.annotation,
            tags.modifier,
            tags.self,
            tags.namespace,
        ],
        color: chalky,
    },
    {
        tag: [
            tags.operator,
            tags.operatorKeyword,
            tags.url,
            tags.escape,
            tags.regexp,
            tags.link,
            tags.special(tags.string),
        ],
        color: cyan,
    },
    { tag: [tags.meta, tags.comment], color: stone },
    { tag: tags.strong, fontWeight: "bold" },
    { tag: tags.emphasis, fontStyle: "italic" },
    { tag: tags.link, color: stone, textDecoration: "underline" },
    { tag: tags.heading, fontWeight: "bold", color: coral },
    {
        tag: [tags.atom, tags.bool, tags.special(tags.variableName)],
        color: whiskey,
    },
    {
        tag: [tags.processingInstruction, tags.string, tags.inserted],
        color: sage,
    },
    { tag: tags.invalid, color: invalid },
]);
/// Extension to enable the One Dark theme (both the editor theme and
/// the highlight style).
const oneDark = [oneDarkTheme, oneDarkHighlightStyle];

/// theme
const themeCompart = new Compartment();
const THEMES = {
    dark: oneDark$1,
    light: oneDark,
};
function reconfigure$1(theme) {
    return themeCompart.reconfigure(THEMES[theme]);
}
function useThemeProp(theme, view) {
    const [compart] = useState(() => themeCompart.of(THEMES[theme]));
    useEffect(() => {
        view === null || view === void 0 ? void 0 : view.dispatch({
            effects: [reconfigure$1(theme)],
        });
    }, [theme, view]);
    return compart;
}

function useDiagnostics(diagnostics, view) {
    useEffect(() => {
        if (view) {
            const founds = translateDiagnostics(diagnostics, view);
            const first = founds[0];
            // scroll into this first diagnostric result
            if (first) {
                view.scrollPosIntoView(first.from);
            }
            view.dispatch(setDiagnostics(view.state, translateDiagnostics(diagnostics, view)));
        }
    }, [diagnostics, view]);
}

/// editable
const extensionsCompart = new Compartment();
function reconfigure(extensions) {
    return extensionsCompart.reconfigure(extensions);
}
function useExtensionsProp(extensions, view) {
    const [compart] = useState(() => extensionsCompart.of(extensions));
    useEffect(() => {
        view === null || view === void 0 ? void 0 : view.dispatch({
            effects: [reconfigure(extensions)],
        });
    }, [extensions, view]);
    return compart;
}

function useChangedValue(value, view) {
    useEffect(() => {
        if (view && !isSameTextAccordingToDoc(view, value)) {
            view.dispatch({
                changes: [
                    {
                        from: 0,
                        to: view.state.doc.length,
                        insert: value,
                    },
                ],
            });
        }
    }, [value, view]);
}
function listenValueChangeAndInvokeCallback(handler) {
    return EditorView.updateListener.of(({ docChanged, state }) => {
        const editable = editableCompart.get(state);
        if (docChanged &&
            handler &&
            editable &&
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            state.facet(editable.facet)) {
            handler(state.doc.toJSON().join(state.lineBreak));
        }
    });
}

/// placeholder
const textOfPlaceholder = Facet.define({
    combine(input) {
        return input[input.length - 1];
    },
});
const placeholderPlguin = ViewPlugin.fromClass(class {
    constructor(view) {
        this.view = view;
        this.dom = this.initPlaceholderMask(view);
        this.updateMaskOverlayer(view.state.facet(textOfPlaceholder));
    }
    update(update) {
        if (update.docChanged && update.state.doc.length === 0) {
            this.updateMaskOverlayer(this.view.state.facet(textOfPlaceholder));
        }
        if (update.state.doc.length !== 0) {
            this.dom.innerHTML = "";
        }
    }
    destory() {
        this.view.dom.removeChild(this.dom);
    }
    initPlaceholderMask(view) {
        const mask = document.createElement("div");
        mask.classList.add("cm-placeholder-mask");
        view.scrollDOM.appendChild(mask);
        return mask;
    }
    updateMaskOverlayer(text) {
        if (typeof text === "string") {
            this.dom.innerText = text;
        }
        else {
            this.dom.innerHTML = text
                .map((str) => {
                return `<div class="cm-line">${str}</div>`;
            })
                .join("");
        }
    }
}, {});
const baseTheme = EditorView.baseTheme({
    ".cm-placeholder-mask": {
        position: "absolute",
        top: 0,
        left: "30px",
        lineHeight: 1.9,
        opacity: 0.4,
    },
});
function placeholder(placeholdertext) {
    return [
        textOfPlaceholder.of(placeholdertext),
        placeholderPlguin,
        baseTheme,
    ];
}

const pluginCompart = new Compartment();
function usePlaceholderProp(text, view) {
    const [compart] = useState(pluginCompart.of(placeholder(text)));
    useEffect(() => {
        view === null || view === void 0 ? void 0 : view.dispatch({
            effects: [pluginCompart.reconfigure(placeholder(text))],
        });
    }, [text, view]);
    return compart;
}

/**
 * 对 codemirror6 的封装
 */
function ReactCodemirror(props, ref) {
    const { onChange } = props;
    const element = useRef();
    const editor = useRef();
    useImperativeHandle(
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    ref, () => ({
        format(configs) {
            startFormat(editor.current, configs);
        },
    }), [editor]);
    const themeCompart = useThemeProp(props.theme || "dark", editor.current);
    const editCompart = useEditableProp(props.editable != undefined ? props.editable : true, editor.current);
    const languageCompart = useLanguageProp(props.language, props.langOptions, editor.current);
    const extensionsCompart = useExtensionsProp(props.extensions || [], editor.current);
    const placeholderCompart = usePlaceholderProp(props.placeholder, editor.current);
    useDiagnostics(props.diagnostics, editor.current);
    useChangedValue(props.value, editor.current);
    useMount(() => {
        const state = EditorState.create({
            doc: props.defaultValue || props.value || "",
            extensions: [
                basicSetup,
                editCompart,
                themeCompart,
                languageCompart,
                extensionsCompart,
                MINIMAP_FLAG ,
                placeholderCompart,
                listenValueChangeAndInvokeCallback(onChange),
            ].filter(Boolean),
        });
        editor.current = new EditorView({
            state: state,
            parent: element.current,
        });
    });
    useUnmount(() => editor.current.destroy());
    return (React.createElement("div", { ref: element, className: "codemirror-editor-body", style: props.style, 
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore 接受css属性，兼容 @emotion/react 的属性
        css: props.css }));
}
var ReactCodemirror$1 = forwardRef(ReactCodemirror);

export { ReactCodemirror$1 as ReactCodemirror };
